/* Copyright (c) 2025 Skyward Experimental Rocketry
 * Authors: Alberto Nidasio, Fabrizio Monti
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <drivers/dma/DMA.h>
#include <miosix.h>
#include <util/util.h>

using namespace miosix;
using namespace Boardcore;

void printBuffer(uint32_t* buffer, size_t size);

int main()
{
    // SCB_DisableDCache(); // sufficiente questo, non necess disabilitare ICache

    DMAStreamGuard stream = DMADriver::instance().acquireStreamForPeripheral(
        DMADefs::Peripherals::PE_MEM_ONLY);

    if (!stream.isValid())
    {
        printf("Error, cannot allocate dma stream\n");
        return 0;
    }

    /**
     * In this test we want to copy a buffer1 into buffer2 with the DMA.
     */

    const int SIZE = 1024;
    // uint8_t buffer1[8] = {1, 2, 3, 4, 5, 6, 7, 8};
    // uint8_t buffer2[8] = {0};
    uint32_t buffer1[SIZE];
    for(int i = 0; i < SIZE; ++i)
        buffer1[i] = i+1;
    uint32_t buffer2[SIZE] = {0};

    printf("Before:\n");
    printf("Buffer 1:\n");
    printBuffer(buffer1, sizeof(buffer1)/sizeof(uint32_t));
    printf("Buffer 2:\n");
    printBuffer(buffer2, sizeof(buffer2)/sizeof(uint32_t));

    DMATransaction trn{
        .direction         = DMATransaction::Direction::MEM_TO_MEM,
        .srcSize           = DMATransaction::DataSize::BITS_8,
        .dstSize           = DMATransaction::DataSize::BITS_8,
        .srcAddress        = buffer1,
        .dstAddress        = buffer2,
        .numberOfDataItems = sizeof(buffer1),
        .srcIncrement      = true,
        .dstIncrement      = true,
        .enableTransferCompleteInterrupt = true,
    };
    stream->setup(trn);
    stream->enable();

    stream->waitForTransferComplete();

    /**
     * con questi 2 funziona, anche se non mi è chiaro il perchè:
     * - la prima linea serve per fare QUASI tutto l'array: copre tutto meno gli ultimi
     * 4-8 bytes
     * - la seconda linea serve a coprire gli ultimi, anche se è sus: gli dico di coprire
     * gli ultimi 4 bytes, ma copre anche gli ultimi 8 se necessario (???)
     * */
    // per uint8_t
    // SCB_InvalidateDCache_by_Addr((uint32_t*)buffer2, SIZE); // risolve solo per i primi 4 byte
    // SCB_InvalidateDCache_by_Addr((uint32_t*)buffer2 + (SIZE/4) - 1, 4); // risolve solo per i primi 4 byte

    // per uint32_t
    SCB_InvalidateDCache_by_Addr((uint32_t*)buffer2, sizeof(buffer2)/*+sizeof(uint32_t)*/); // risolve solo per i primi 4 byte
    SCB_InvalidateDCache_by_Addr((uint32_t*)buffer2 + (SIZE) - 1, 4); // risolve solo per i primi 4 byte


    // SCB_CleanDCache_by_Addr((uint32_t*)buffer2, 8); // non fa nulla


    printf("--- After:\n");
    printf("Buffer 1:\n");
    printBuffer(buffer1, sizeof(buffer1)/sizeof(uint32_t));
    printf("Buffer 2:\n");
    printBuffer(buffer2, sizeof(buffer2)/sizeof(uint32_t));

    return 0;
}

void printBuffer(uint32_t* buffer, size_t size)
{
    for (size_t i = 0; i < size - 1; i++)
        printf("%d,", buffer[i]);

    printf("%d\n", buffer[size - 1]);
}
